
# shlibs is Copyright (C) 2020 Nicolae Iotu, nicolae.g.iotu@gmail.com
# https://shlibs.org, https://shlibs.net 
# License: Apache-2.0 modified with Convergence Terms (LICENSE-section 10)
# "Use for free. Contribute rather than diverge."

# Author Nicolae Iotu, nicolae.g.iotu@gmail.com

# TAGS: PORTABLE, MORPH


if type s_err >/dev/null 2>&1 ; then :
else
	. "${shlibs_dirpath}/var/comp/ptbl/shlibs_err.sh"
fi
if ( echo ${shlibs_redir_vars:?} ) >/dev/null 2>&1 ; then :
else
	. "${shlibs_dirpath}/var/comp/ptbl/shlibs_redir.sh"
	if [ "${shlibs_redir_vars_flag}" = '0' ]; then
		unset -v shlibs_redir_vars_flag
		if [ "${SHLIBS_COMPLEX_PIPE}" = '0' ]; then
			unset -v SHLIBS_COMPLEX_PIPE
			"${shlibs_dirpath}"/shlibs "${@}" ${shlibs_redir_vars}
		else
			unset -v SHLIBS_COMPLEX_PIPE
			"${shlibs_dirpath}"/shlibs "${@}" "${shlibs_redir_vars}"
		fi
		exit ${?}
	fi
fi
rl_get_script_path=1
while echo "${1}" | ${SHLIBS_GREP} '^-.$' >/dev/null 2>&1 
do
	case ${1} in
		-z)
			opts_block_default_function=0
			export opts_block_default_function
			shift
			;;
		-v)
			if [ ${#} -gt 2 ]; then
				opts_lib_version=${2}
				test ${opts_lib_version} -gt 0 2>/dev/null || {
					s_err 'Invalid version number ( > 0 ).'
					exit 1
				}
				export opts_lib_version
				shift 2
			elif [ ${#} -gt 1 ]; then
				s_err 'No lib specified!'
				exit 1
			else
				echo "${SHLIBS_VERSION}"
				exit
			fi
			;;
		-p)
			rl_get_script_path=0
			shift
			;;
		-h)
			opts_block_default_function=0
			opts_help=0
			shift
			;;
		-x)
			opts_block_default_function=0
			opts_examples=0
			shift
			;;
		*)
			s_err "Incompatible option ${1}."
			exit 1
			;;
	esac	
done
if [ ${#} -gt 0 ]; then
	rl_dev_code="${1}"
	shift
else
	s_err 'Missing library code!'
	exit 1
fi
if [ ${#} -gt 20 ]; then
	s_err 'Maximum number of params (20) exceeded.'
	exit 1
fi
rl_count_libs_found=0 rl_force_section=1 rl_section=''
rl_post_find_preps() {
	if [ -n "${rl_lib_path}" ]; then
		rl_lib_file="${rl_lib_path##*/}"
		rl_lib_file_name=${rl_lib_file%.*}
		rl_lib_file_ext=${rl_lib_file##*.}
		if [ "${rl_section}" = 'official' ]; then
			if [ "${rl_lib_file_ext}" = 'sh' ] && \
				[ "${#rl_lib_file_name}" -gt 1 ]; then
				return
			fi
		else
			return
		fi
	fi
	rl_count_libs_found=0
}
rl_find() {
	rl_lib_find_path="${shlibs_dirpath}"/libs
	if [ ${rl_force_section} -eq 0 ]; then
		rl_force_section=1
	else
		rl_lib_find_path="${rl_lib_find_path}/${rl_section}"
	fi
	rl_dev_dir=$(dirname -- "${1}")
	IFS='/'
	for rl_slash_seq in ${rl_dev_dir}
	do	
		if [ -n "${rl_slash_seq}" ] && [ "${rl_slash_seq}" != '.' ]; then
			rl_lib_find_path="${rl_lib_find_path}/${rl_slash_seq}"
			if [ -h "${rl_lib_find_path}" ]; then
				rl_lib_find_path=$( cd "${rl_lib_find_path}" ; pwd -L ) || err_fs
			fi
		fi
	done
	IFS=${o_ifs}
	rl_fl="${1##*/}"
	if [ "${rl_section}" = 'official' ]; then
		rl_fl=${1%.*}
		rl_fl="${rl_fl}.sh"
	else
		if echo "${1}" | ${SHLIBS_GREP} '/' >/dev/null 2>&1 ; then :
		else
			rl_fl="${rl_fl}*"
		fi
	fi
	if [ -d "${rl_lib_find_path}" ]; then
		if [ -d "${rl_lib_find_path}/src" ] && [ -d "${rl_lib_find_path}/headers" ]; then
			rl_lib_src_path="${rl_lib_find_path}/src"
			rl_lib_headers_path="${rl_lib_find_path}/headers"
		else
			rl_lib_src_path="${rl_lib_find_path}"
			rl_lib_headers_path=''
		fi
		rl_lib_path=$(find -L "${rl_lib_src_path}" -type f \
			-name "${rl_fl#/}" ! -name "*.dat")
	else
		return
	fi
	rl_count_libs_found=$(echo "${rl_lib_path}" | wc -l | xargs echo)	
	if [ ${rl_count_libs_found} -eq 1 ]; then
		rl_post_find_preps
	elif [ ${rl_count_libs_found} -gt 1 ]; then
		if [ -n "${rl_lib_headers_path}" ]; then
			rl_test_err=$(echo "${rl_lib_path}" | xargs dirname -- | uniq \
				| wc -l | xargs echo)
			if [ ${rl_test_err} -ne ${rl_count_libs_found} ]; then
				s_err "Duplicate library names found for '${rl_dev_code}': 
${rl_lib_path}"
				exit 1
			fi
			unset -v rl_test_err
			IFS=${_nl}
			rl_mvers=0
			rl_find_index=0
			for rl_mvers_lib in ${rl_lib_path}
			do
				rl_find_index=$((rl_find_index+1))
				rl_mvers_lib=$(echo "${rl_mvers_lib}" | \
				${SHLIBS_AWK} '{
					match($0, /.\/libs\/(official|dev|community)\/src\//);
					if (RSTART > 0) {
						m=substr($0, RSTART, RLENGTH);
						sub("src","headers",m);
						print substr($0, 1, RSTART-1)m substr($0, RSTART+RLENGTH); 
					} else {
						print "";
					}
				}')
				rl_mvers_lib=${rl_mvers_lib%.*}
				if [ -r "${rl_mvers_lib}" ]; then
					rl_ivers=$(${SHLIBS_AWK} \
						'{ match($0,/r_l_s:/); \
						if (RSTART > 0) { print substr($0, RSTART+7); } }' \
						< "${rl_mvers_lib}" | xargs)
					if [ -n "${rl_ivers}" ]; then
						if [ -n "${opts_lib_version}" ] && \
							[ ${opts_lib_version} -gt 0 ] && \
							[ ${opts_lib_version} -eq ${rl_ivers} ]; then
							rl_mvers=${opts_lib_version}
							break
						elif [ ${rl_ivers} -gt ${rl_mvers} ]; then
							rl_mvers=${rl_ivers}
						fi
					fi
				fi
			done
			IFS=${o_ifs}
			if [ ${rl_mvers} -gt 0 ]; then
				rl_lib_path=$(echo "${rl_lib_path}" | sed "${rl_find_index}"' !d')
				rl_count_libs_found=1
				rl_post_find_preps
			else
				s_err "Couldn't find a version number for ${rl_dev_code}"
				exit 1
			fi
		else
			return
		fi
	fi
}
rl_specific_parse_count=0
rl_specific_section=${rl_dev_code#/}
rl_specific_section=${rl_specific_section%%/*}
rl_specific_libcode=${rl_dev_code#*/}
if [ "${rl_specific_section}" = "${rl_specific_libcode}" ] &&
	[ ${#rl_dev_code} -eq ${#rl_specific_libcode} ]; then
	rl_specific_parse_count=1
else
	IFS='/'
	for rl_section in ${rl_dev_code#/}
	do	
		rl_section=$(echo ${rl_section} | xargs)
		if [ -z "${rl_section}" ] || [ "${rl_section}" = '.' ] || \
			[ "${rl_section}" = '..' ]; then		
			s_err "Invalid libcode. Use: shlibs section/libcode."
			exit 1
		fi
		rl_specific_parse_count=$((rl_specific_parse_count+1))
	done
	IFS=${o_ifs}
fi
if [ ${rl_specific_parse_count} -gt 1 ]; then
	case "${rl_specific_section}" in
		official|dev|community)
			rl_lib_path=$(echo "./libs/${rl_dev_code}" \
				| sed 's/[\/]\{2,\}/\//g')
			if [ -e "${rl_lib_path}" ]; then
				rl_count_libs_found=1
				rl_lib_file="${rl_lib_path##*/}"
				rl_lib_file_name=${rl_lib_file%.*}
			else
				rl_force_section=0
				rl_find "${rl_dev_code}"
			fi
			;;
		*)
			s_err "Unknown section '${rl_specific_section}'.
To setup a script use: 'shlibs -s /path/to/script'"
			exit 1
			;;
	esac
elif [ ${rl_specific_parse_count} -eq 1 ]; then
	if [ ${rl_count_libs_found} -eq 0 ]; then
		rl_section='official'
		rl_find "${rl_dev_code}"
	fi
	if [ ${rl_count_libs_found} -eq 0 ]; then
		rl_section='dev'
		rl_find "${rl_dev_code}"
	fi
	if [ ${rl_count_libs_found} -eq 0 ]; then
		rl_section='community'
		rl_find "${rl_dev_code}"
	fi
else
	s_err 'Unusual request. Use: shlibs section/libcode.'
	exit 1
fi
if [ ${rl_count_libs_found} -ne 1 ]; then
	s_err "Error: Found ${rl_count_libs_found} libraries for code '${rl_dev_code}'.
Use 'shlibs' to search for a library code first." 
	exit 1
fi
rl_lib_dirpath=$( dirname -- "${rl_lib_path}" )
if [ ${rl_get_script_path} -eq 0 ]; then
	p_path=$( cd "${rl_lib_dirpath}" ; pwd -L ) || err_fs
	p_path_bn="${rl_lib_path##*/}"
	echo "${p_path}/${p_path_bn}"
	exit
fi
rl_restore_lang() {
	if [ ${SHLIBS_HAS_SETENV} -eq 0  ]; then
		setenv LANG=${SHLIBS_ORIG_LC_MESSAGES}
	else
		LANG=${SHLIBS_ORIG_LC_MESSAGES}
		export LANG
	fi
}
if [ "x${rl_cleanup_vars}" = 'x0' ]; then
	unset -v sg_lib_format SHLIBS_CLEANUP_DISPLAY SHLIBS_SHOW_VERSION_IN_QUERY \
		SHLIBS_MATCH_HIGHLIGHT SHLIBS_MATCH_MAX SHLIBS_MATCH_PAGE_SIZE \
		SHLIBS_MAX_RESULTS_MEM SHLIBS_SHOW_OPTIONS SHLIBS_FORCE_SHELL \
		SHLIBS_TERMINAL_CHAR_WIDTH \
		rl_specific_parse_count rl_get_script_path rl_count_libs_found \
		rl_find_index rl_slash_seq rl_cleanup_vars shlibs_fb shlibs_redir_vars \
		shlibs_redir_vars_flag shlibs_redirect rl_test_lib_wording_output		
fi
if [ -f "${rl_lib_path}" ] && [ -r "${rl_lib_path}" ]; then
	rl_dev_code_bn=$( echo "${rl_dev_code##*/}" | \
		sed -e 's/[^a-zA-Z0-9\./_-]//g' )
	rl_dev_code_bn="${rl_dev_code_bn%%.*}"
	rl_var_script_return_code=0
	if ({	
			if [ "${opts_help}" != '0' ] && \
				[ "${opts_examples}" != '0' ]; then
				rl_test_lib_wording_output='2>/dev/null'
			else
				rl_test_lib_wording_output='>/dev/null 2>&1'
			fi
			rl_h1_interpreter=$(head -n 1 "${rl_lib_path}")
			if echo "${rl_h1_interpreter}" | \
				${SHLIBS_GREP} '^#!.\{0,\}/bin/.\{0,2\}sh$' >/dev/null ; then
				. "${rl_lib_path}" "${rl_test_lib_wording_output}"
			else
				return 255 2>/dev/null
			fi
			rl_plc=$(echo ${SHLIBS_ORIG_LC_MESSAGES} | wc -l | xargs echo)
			if [ -n "${SHLIBS_ORIG_LC_MESSAGES}" ] && [ ${rl_plc} -eq 1 ]; then :
			else
				if [ ${SHLIBS_HAS_SETENV} -eq 0 ] ; then
					setenv LANG=POSIX
				else
					LANG=POSIX
					export LANG
				fi
			fi
			if [ "${rl_cleanup_vars}" = '0' ]; then
				unset -v rl_plc
			fi
			if [ "${opts_block_default_function}" != '0' ]; then
				if type "${rl_dev_code_bn}" 2>/dev/null | \
					${SHLIBS_GREP} 'function' >/dev/null 2>&1 ; then
					rl_restore_lang
					cd "${shlibs_cwd}"
					if ${rl_dev_code_bn} "${@}" ; then
						rl_var_script_return_code=0
					else
						return ${?} 2>/dev/null
					fi
				else
					rl_restore_lang
					s_err "Warning: Can't find a function \
'${rl_dev_code_bn}' in file '${rl_lib_path#\./}'.
You can use 'libpath=\$( shlibs -p \"${rl_dev_code}\" ); . \"\${libpath}\"' 
to source '${rl_dev_code_bn}' and call functions within as required."
					return 1 2>/dev/null
				fi
			fi
			if [ "${opts_help}" = '0' ] \
				|| [ "${dq_lib_help}" = '0' ]; then
				if type "${rl_dev_code_bn}_help" 2>/dev/null | \
					${SHLIBS_GREP} 'function' >/dev/null 2>&1 ; then
					cd "${shlibs_cwd}"
					if [ "${dq_lib_help}" = '0' ]; then
						eval "${rl_dev_code_bn}_help" "${@}" \
							> "${shlibs_dirpath}/var/tmp/${shlibs_session}/dq_libhelp"
					else
						eval "${rl_dev_code_bn}_help" "${@}"
					fi
					rl_var_script_return_code=${?}
					cd "${shlibs_dirpath}"
				else
					rl_var_script_return_code=1
					echo "No help available for '${rl_dev_code}'.
Missing '${rl_dev_code_bn}_help' function."
				fi
				rl_restore_lang
				exit ${rl_var_script_return_code}
			fi
			if [ "${opts_examples}" = '0' ] \
				|| [ "${dq_lib_examples}" = '0' ]; then
				if type "${rl_dev_code_bn}_examples" 2>/dev/null | \
					${SHLIBS_GREP} 'function' >/dev/null 2>&1 ; then
					cd "${shlibs_cwd}"
					if [ "${dq_lib_examples}" = '0' ]; then
						eval "${rl_dev_code_bn}_examples" "${@}" \
							> "${shlibs_dirpath}/var/tmp/${shlibs_session}/dq_libexamples"
					else
						eval "${rl_dev_code_bn}_examples" "${@}"
					fi
					rl_var_script_return_code=${?}
					cd "${shlibs_dirpath}"					
				else
					rl_var_script_return_code=1
					echo "No examples available for '${rl_dev_code}'.
Missing '${rl_dev_code_bn}_examples' function."
				fi
				rl_restore_lang
				exit ${rl_var_script_return_code}
			fi
		}) ; then
		cd "${shlibs_dirpath}"
		exit ${rl_var_script_return_code}
	else
		SHLIBS_SCRIPT_EXIT_CODE=${?}
		if [ ${SHLIBS_SCRIPT_EXIT_CODE} -eq 255 ]; then
			if [ -x "${rl_lib_path}" ]; then
				rl_remove_x=1
			else
				if chmod u+x "${rl_lib_path}" ; then
					if [ -x "${rl_lib_path}" ]; then
						rl_remove_x=0
					else
						s_err "Error: add +x for script '${rl_lib_path}' \
, or elevate privileges and try again."
						exit 1
					fi
				else
					s_err "Error: add +x for script '${rl_lib_path}' \
, or elevate privileges and try again."
					exit 1
				fi
			fi
			cd "${shlibs_cwd}"
			if [ ${#} -gt 0 ]; then
				"${rl_lib_path}" "${@}"
			else
				"${rl_lib_path}"
			fi
			rl_var_script_return_code=${?}
			cd "${shlibs_dirpath}"
			if [ ${rl_remove_x} -eq 0 ]; then
				chmod u-x "${rl_lib_path}"
			fi
			return ${rl_var_script_return_code} 2>/dev/null			
		else
			cd "${shlibs_dirpath}"
			exit ${SHLIBS_SCRIPT_EXIT_CODE}
		fi
	fi
else
	rl_error_content="'${rl_lib_path}' is not a regular-readable file."
	s_err "${rl_error_content}"
	exit 1
fi
